@@ -1,5 +1,5 @@
 package JSON::Karabiner::Manipulator::Actions ;
-$JSON::Karabiner::Manipulator::Actions::VERSION = '0.016';
+$JSON::Karabiner::Manipulator::Actions::VERSION = '0.017';
 use strict;
 use warnings;
 use JSON;
@@ -47,14 +47,6 @@ sub _is_exclusive {
   $s->{$property} = 1;
 }
 
-sub TO_JSON {
-  my $obj = shift;
-  my $name = $obj->{def_name};
-  my $value = $obj->{data};
-  return { $name => $value };
-
-  # return { %{ shift() } };
-}
 
 # ABSTRACT: parent class for action classes
 
@@ -86,7 +78,7 @@ for reference purposes only.
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 AUTHOR
 
