@@ -1,5 +1,5 @@
 package JSON::Karabiner::Manipulator::Actions::To ;
-$JSON::Karabiner::Manipulator::Actions::To::VERSION = '0.016';
+$JSON::Karabiner::Manipulator::Actions::To::VERSION = '0.017';
 use strict;
 use warnings;
 use JSON;
@@ -110,6 +110,10 @@ sub add_set_variable {
   my $value = shift;
   croak 'No value passed' unless defined $value;
 
+  if ($value =~ /^\d+$/) {
+    $value = $value + 0;
+  }
+
   my %hash;
   $hash{set_variable}{name} = $name;
   $hash{set_variable}{value} = $value;
@@ -268,7 +272,7 @@ individually and then add your modifiers.
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 AUTHOR
 
