@@ -1,5 +1,5 @@
 package JSON::Karabiner::Manipulator::Actions::To_delayed_if_canceled ;
-$JSON::Karabiner::Manipulator::Actions::To_delayed_if_canceled::VERSION = '0.016';
+$JSON::Karabiner::Manipulator::Actions::To_delayed_if_canceled::VERSION = '0.017';
 use strict;
 use warnings;
 use JSON;
@@ -15,8 +15,6 @@ sub new {
   }
   my $obj;
   if ($main::has_delayed_action) {
-    use Data::Dumper qw(Dumper);
-    print Dumper 'kajskfj';
     $obj = $main::has_delayed_action;
   } else {
     $obj = $class->SUPER::new('to_delayed_action', $value);
@@ -48,7 +46,7 @@ JSON::Karabiner::Manipulator::Actions::To_delayed_if_canceled - to_delayed_if_ca
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 AUTHOR
 
