@@ -1,5 +1,5 @@
 package JSON::Karabiner::Manipulator::Conditions::Variable_if ;
-$JSON::Karabiner::Manipulator::Conditions::Variable_if::VERSION = '0.016';
+$JSON::Karabiner::Manipulator::Conditions::Variable_if::VERSION = '0.017';
 use strict;
 use warnings;
 use JSON;
@@ -21,6 +21,9 @@ sub add_variable {
   croak 'A variable name is required' unless $name;
   croak 'A value for the varaible name is required' unless $value;
   #TODO: Validates keys
+  if ($value =~ /^\d+$/) {
+    $value = $value + 0;
+  }
   $s->{data}{name} = $name;
   $s->{data}{value} = $value;
 
@@ -66,7 +69,7 @@ JSON::Karabiner::Manipulator::Conditions::Variable_if - definition for Frontmost
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 AUTHOR
 
